#   Copyright (C) 1987-1999 by Jeffery P. Hansen
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Last edit by hansen on Tue May 11 10:23:06 1999
#

#
# Note on bindtags usage - In tcl/tk bindings for multi key sequences
# are executed regardless of overlap with other bindings.  For example,
# if "Ctl-X Ctl-S" and "Ctl-S g" are commands, then the key sequence
# "Ctl-X Ctl-S g" will cause commands to be executed.  To avoid this
# confusing behavior, a special nop command key are used.  After the
# end of a two-key sequence, a special key not used in any commands is
# forced into the event stream to break up any unexpected commands. 
#
# Another problem is when "g" is a command, but "Ctl-X g" is not a
# command.  Since "Ctl-X" is a command prefix, if the next character
# does not form a valid mult-charcter command, it should be ignored.
# We implement this using bindtags on windows.  Single charcter commands
# are bound to the tag "keywin", and multi-character commands are bound
# to the tag "key2win".  These tags are attached to all windows where
# which process commands.  When a prefix character is entered, the
# "keywin" tag is temporarily removed from all windows, and restored after
# a second character has been entered. 
#

set bind_win_list [list]

proc endPrefix {} {
  global bind_win_list
  foreach w $bind_win_list {
    catch {
      set bt [bindtags $w]
      if { [lsearch $bt keywin] < 0 } {
	bindtags $w [concat keywin $bt]
      }
    }
  }
  event generate . <Meta-KeyPress-U>
}

proc startPrefix {} {
  global bind_win_list
  foreach w $bind_win_list {
    catch { bindtags $w [ldelete [bindtags $w] keywin] }
  }
}

proc tkg_setBindings {w} {
  global bind_win_list
  if { [lsearch $bind_win_list $w] < 0 } {
    append bind_win_list $w
    bindtags $w [concat keywin key2win [bindtags $w]]
  }
}

bind keywin <a>				{ gat_make and }
bind keywin <A>				{ gat_make and -invert Z}
bind keywin <o>				{ gat_make or }
bind keywin <O>				{ gat_make or -invert Z}
bind keywin <KeyPress-x>		{ gat_make xor }
bind keywin <KeyPress-X>		{ gat_make xor -invert Z}
bind keywin <s>				{ gat_make switch }
bind keywin <I>				{ gat_make buf -invert Z }
bind keywin <t>				{ gat_make bufif1 }
bind keywin <T>				{ gat_make tty }
bind keywin <b>				{ gat_make buf }
bind keywin <B>				{ gat_make block }
bind keywin <braceleft>			{ gat_make output }
bind keywin <braceright>		{ gat_make input }
bind keywin <equal>			{ gat_make inout  }
bind keywin <plus>			{ gat_make add }
bind keywin <asterisk>			{ gat_make mult }
bind keywin <slash>			{ gat_make divide }
bind keywin <c>				{ gat_make clock }
bind keywin <R>				{ gat_make ram }
bind keywin <u>				{ gat_make rom }
bind keywin <d>				{ gat_make dip }
bind keywin <v>				{ gat_make supply1 }
bind keywin <g>				{ gat_make supply0 }
bind keywin <r>				{ gat_make register }
bind keywin <S>L			{ gat_make lshift }
bind keywin <S>R			{ gat_make rshift }
bind keywin <S>A			{ gat_make arshift }
bind keywin <S>O			{ gat_make roll }
bind keywin <w>				{ gat_make concat }
bind keywin <C>				{ gat_make comment }
bind keywin <F>				{ gat_make frame }

bind keywin <m>				{ gat_make mux }
bind key2win <M>2			{ gat_make mux; endPrefix }
bind key2win <M>3			{ gat_make mux -pins I=3; endPrefix }
bind key2win <M>4			{ gat_make mux -pins I=4; endPrefix }
bind key2win <M>5			{ gat_make mux -pins I=5; endPrefix }
bind key2win <M>6			{ gat_make mux -pins I=6; endPrefix }
bind key2win <M>7			{ gat_make mux -pins I=7; endPrefix }
bind key2win <M>8			{ gat_make mux -pins I=8; endPrefix }
bind key2win <M>9			{ gat_make mux -pins I=9; endPrefix }
bind key2win <M>0			{ gat_make mux -pins I=10; endPrefix }

bind key2win <D>2			{ gat_make demux; endPrefix }
bind key2win <D>3			{ gat_make demux -pins Z=3; endPrefix }
bind key2win <D>4			{ gat_make demux -pins Z=4; endPrefix }
bind key2win <D>5			{ gat_make demux -pins Z=5; endPrefix }
bind key2win <D>6			{ gat_make demux -pins Z=6; endPrefix }
bind key2win <D>7			{ gat_make demux -pins Z=7; endPrefix }
bind key2win <D>8			{ gat_make demux -pins Z=8; endPrefix }
bind key2win <D>9			{ gat_make demux -pins Z=9; endPrefix }
bind key2win <D>0			{ gat_make demux -pins Z=10; endPrefix }

bind key2win <Control-r>a		{ gat_make and -pins I=1; endPrefix }
bind key2win <Control-r>A		{ gat_make and -pins I=1 -invert Z; endPrefix }
bind key2win <Control-r>o		{ gat_make or -pins I=1; endPrefix }
bind key2win <Control-r>O		{ gat_make or -pins I=1 -invert Z; endPrefix }
bind key2win <Control-r>x		{ gat_make xor -pins I=1; endPrefix }
bind key2win <Control-r>X		{ gat_make xor -pins I=1 -invert Z; endPrefix }

bind key2win <Control-t>n		{ gat_make nmos; endPrefix }
bind key2win <Control-t>p		{ gat_make pmos; endPrefix }

bind keywin <bracketleft>		{ gat_addIn }
bind keywin <bracketright>		{ gat_addOut }
bind keywin <minus>			{ gat_addInOut }
bind keywin <at>			{ gat_changePinDir }

bind keywin <F1>			{ tkg_setMode 1 }
bind keywin <F2>			{ tkg_setMode 3 }
bind keywin <F3>			{ tkg_setMode 0 }
bind keywin <F4>			{ tkg_setMode 2 }
bind keywin <F5>			{ tkg_setMode 9 }
if { $tkg_allowDebug } {  bind keywin <F6> { tkg_setMode 12 } }

bind keywin <Control-F1>		{ set rot 0 }
bind keywin <Control-F2>		{ set rot 90 }
bind keywin <Control-F3>		{ set rot 180 }
bind keywin <Control-F4>		{ set rot 270 }

bind keywin <greater>			{ gat_openBox }
bind keywin <less>			{ gat_closeBox }
bind keywin <Control-v>			{ gat_replicate }
bind keywin <Delete>			{ gat_deleteSelected }
bind keywin <E>				{ gat_editProps }

bind keywin <p>				{ tkg_errBoxUp }
bind keywin <Control-p>			{ tkg_errBoxUp }
bind keywin <n>				{ tkg_errBoxDown }
bind keywin <Control-n>			{ tkg_errBoxDown }

bind key2win <Control-x>k		{ tkg_new; endPrefix }
bind key2win <Control-x><Control-f>	{ tkg_load; endPrefix }
bind key2win <Control-x><Control-l>	{ tkg_loadLibrary; endPrefix }
bind key2win <Control-x><Control-s>	{ tkg_saveCurrent; endPrefix }
bind key2win <Control-x><Control-w>	{ tkg_save; endPrefix }
bind key2win <Control-x><Control-p>	{ tkg_printDlg; endPrefix }
bind key2win <Control-x><Control-c>	{ tkg_exit; endPrefix }
bind key2win <Control-x>o		{ tkg_editOptions; endPrefix }
bind key2win <Control-x>x		{ gat_copyToBuf; endPrefix }
bind key2win <Control-x>a		{ gat_selectAll; endPrefix }
bind key2win <Control-x>E		{ tkg_editCircProp; endPrefix }
bind key2win <Control-x>f		{ gat_anchor 1; endPrefix }
bind key2win <Control-x>u		{ gat_anchor 0; endPrefix }

bind keywin <Control-w>			{ gat_cutToBuf }
bind keywin <Control-y>			{ gat_yankFromBuf }
bind keywin <Control-f>			{ tkg_findObject }

bind key2win <Control-s>b		{ gat_simulate; endPrefix }
bind key2win <Control-s>e		{ gat_endSimulate; endPrefix }
bind key2win <Control-s>g		{ tkg_simRun; endPrefix }
bind key2win <Control-s>s		{ tkg_simStop; endPrefix }
bind key2win <Control-s>k		{ tkg_editBreakpoints; endPrefix }
bind key2win <Control-s>x		{ tkg_doSimScript; endPrefix }
bind key2win <Control-s>l		{ tkg_simLoadMem; endPrefix }
bind key2win <Control-s>d		{ tkg_simDumpMem; endPrefix }
bind key2win <Control-s>p		{ tkg_scopePrintDlg; endPrefix }
bind keywin <space>			{ tkg_simStep }
bind keywin <Tab>			{ tkg_simCycle }

bind key2win <Control-b>n		{ tkg_blockNew; endPrefix }
bind key2win <Control-b>d		{ tkg_blockDelete; endPrefix }
bind key2win <Control-b>c		{ tkg_blockCopy; endPrefix }
bind key2win <Control-b>r		{ tkg_blockRename; endPrefix }
bind key2win <Control-b>s		{ gat_setBlockDesc; endPrefix }
bind key2win <Control-b>e		{ gat_editBlockDesc; endPrefix }

bind keywin <Left>			{ gat_moveGate -1 0 }
bind keywin <Right>			{ gat_moveGate 1 0 }
bind keywin <Up>			{ gat_moveGate 0 -1 }
bind keywin <Down>			{ gat_moveGate 0 1 }

bind keywin <Control-g>			{ endPrefix; tkg_cancel; gat_unselectGates }
bind keywin <Meta-KeyPress-U>		{ }
bind keywin <Escape>			{ tkg_cancel; gat_unselectGates }

######################################################################
#
# The plain bindings seem to be activiated even for Alt and Control
# modified keys.  Disable some of the problem   bindings here.
#
bind keywin <Control-x>			{ startPrefix }
bind keywin <Control-b>			{ startPrefix }
bind keywin <Control-s>			{ startPrefix }
bind keywin <Control-t>			{ startPrefix }
bind keywin <Control-r>			{ startPrefix }
bind keywin <M>				{ startPrefix }
bind keywin <D>				{ startPrefix }
bind keywin <Alt-m>			{ }
bind keywin <Alt-t>			{ }

bind key2win <Control-x><KeyPress>	{ if { [string compare %A ""] != 0 } { endPrefix } }
bind key2win <Control-b><KeyPress>	{ if { [string compare %A ""] != 0 } { endPrefix } }
bind key2win <Control-s><KeyPress>	{ if { [string compare %A ""] != 0 } { endPrefix } }
bind key2win <Control-t><KeyPress>	{ if { [string compare %A ""] != 0 } { endPrefix } }
bind key2win <Control-r><KeyPress>	{ if { [string compare %A ""] != 0 } { endPrefix } }
bind key2win <M><KeyPress>		{ if { [string compare %A ""] != 0 } { endPrefix } }
bind key2win <D><KeyPress>		{ if { [string compare %A ""] != 0 } { endPrefix } }


